#include "support.h"

hatval hat;

void calibrate(void);
unsigned int getscan(void);

void main(void) {
	FILE *config = NULL;
	int x = 0, y = 0, z = 0, i = 0;

	clrscr();

	config = fopen("TMLoader.cfg","rb");
	if (config != NULL) {
		fread(&hat,sizeof(hatval),1,config);
		fclose(config);
		config = NULL;
		++x;
	} else {++y;++z;}

	if (x == 1) {
		printf("Configuration file found and loaded.\r\n");
		printf("Would you like to reprogram the coolie hat functions? ");
		i = 0;
		while(i == 0) {
			i = getch();
			if (i == 0) getch();
			if (i == 'y') i = 'Y';
			if (i == 'n') i = 'N';
			if ((i != 'Y') && (i != 'N')) i = 0;
		}
		printf("%c\r\n",i);
		if (i == 'Y') ++y;
		printf("Would you like to recalibrate the coolie hat? ");
		i = 0;
		while(i == 0) {
			i = getch();
			if (i == 0) getch();
			if (i == 'y') i = 'Y';
			if (i == 'n') i = 'N';
			if ((i != 'Y') && (i != 'N')) i = 0;
		}
		printf("%c\r\n",i);
		if (i == 'Y') ++z;
	}

	if (y == 1) {
		printf("\r\n   Press the key to be mapped to UP: ");
		while(kbhit()) if (getch() == 0) getch();
		i = getscan();
		hat.uscan = (i & 0xff00) >> 8;
		hat.uchar = i & 0x00ff;
		printf("%c (0x%04X)\r\n",hat.uchar,i);

		printf("Press the key to be mapped to RIGHT: ");
		while(kbhit()) if (getch() == 0) getch();
		i = getscan();
		hat.rscan = (i & 0xff00) >> 8;
		hat.rchar = i & 0x00ff;
		printf("%c (0x%04X)\r\n",hat.rchar,i);

		printf(" Press the key to be mapped to DOWN: ");
		while(kbhit()) if (getch() == 0) getch();
		i = getscan();
		hat.dscan = (i & 0xff00) >> 8;
		hat.dchar = i & 0x00ff;
		printf("%c (0x%04X)\r\n",hat.dchar,i);

		printf(" Press the key to be mapped to LEFT: ");
		while(kbhit()) if (getch() == 0) getch();
		i = getscan();
		hat.lscan = (i & 0xff00) >> 8;
		hat.lchar = i & 0x00ff;
		printf("%c (0x%04X)\r\n",hat.lchar,i);
	}

	if (z == 1) calibrate();

	config = fopen("tmloader.cfg","w+b");
	if(config == NULL) {
		printf("\r\nUnable to save configuraton.\r\n");
		return;
	}
	fwrite(&hat,sizeof(hatval),1,config);
	fclose(config);

	return;
}

void calibrate(void) {
	int count,hold;
	char x;

	printf("\r\nMove hat around then release.\r\n");
	printf("When value stabalizes, press any key:      ");
	while(!kbhit()) {
		disable();
		outportb(0x201,0xff);
		for(count = 0;count < 9999;count++) {
			x = inportb(0x201) & 8;
			if (x == 0) {
				hold = count;
				count = 9999;
			}
		}
		enable();
		if (hold > hat.max) hat.max = hold;
		printf("\b\b\b\b\b%5i",hold);
	}
	if(getch() == 0) getch();
	hat.center = hold;

	printf("\r\nPush and hold the hat in the UP position.\r\n");
	printf("When value stabalizes, press any key:      ");
	while(!kbhit()) {
		disable();
		outportb(0x201,0xff);
		for(count = 0;count < 9999;count++) {
			x = inportb(0x201) & 8;
			if (x == 0) {
				hold = count;
				count = 9999;
			}
		}
		enable();
		if (hold > hat.max) hat.max = hold;
		printf("\b\b\b\b\b%5i",hold);
	}
	if(getch() == 0) getch();
	hat.up = hold;

	printf("\r\nPush and hold the hat in the RIGHT position.\r\n");
	printf("When value stabalizes, press any key:      ");
	while(!kbhit()) {
		disable();
		outportb(0x201,0xff);
		for(count = 0;count < 9999;count++) {
			x = inportb(0x201) & 8;
			if (x == 0) {
				hold = count;
				count = 9999;
			}
		}
		enable();
		if (hold > hat.max) hat.max = hold;
		printf("\b\b\b\b\b%5i",hold);
	}
	if(getch() == 0) getch();
	hat.right = hold;

	printf("\r\nPush and hold the hat in the DOWN position.\r\n");
	printf("When value stabalizes, press any key:      ");
	while(!kbhit()) {
		disable();
		outportb(0x201,0xff);
		for(count = 0;count < 9999;count++) {
			x = inportb(0x201) & 8;
			if (x == 0) {
				hold = count;
				count = 9999;
			}
		}
		enable();
		if (hold > hat.max) hat.max = hold;
		printf("\b\b\b\b\b%5i",hold);
	}
	if(getch() == 0) getch();
	hat.down = hold;

	printf("\r\nPush and hold the hat in the LEFT position.\r\n");
	printf("When value stabalizes, press any key:      ");
	while(!kbhit()) {
		disable();
		outportb(0x201,0xff);
		for(count = 0;count < 9999;count++) {
			x = inportb(0x201) & 8;
			if (x == 0) {
				hold = count;
				count = 9999;
			}
		}
		enable();
		if (hold > hat.max) hat.max = hold;
		printf("\b\b\b\b\b%5i",hold);
	}
	if(getch() == 0) getch();
	hat.left = hold;

	hat.diff = hat.right - hat.up;
	if ((hat.down - hat.right) < hat.diff)
		hat.diff = hat.down - hat.right;
	if ((hat.left - hat.down) < hat.diff)
		hat.diff = hat.left - hat.down;
	if ((hat.center - hat.left) < hat.diff)
		hat.diff = hat.center - hat.left;
	hat.diff /= 3;

}

unsigned int getscan(void) {
	unsigned int keyscan;
	asm {
		push ax
		push bx
		push cx
		push dx
		mov ah,0x10
		int 16h
		mov [keyscan],ax
		pop dx
		pop cx
		pop bx
		pop ax
	}
	return keyscan;
}